import { createToken, createPlugin, memoize } from 'fusion-core';
import bodyParser from 'koa-bodyparser';
import { matchPath } from 'fusion-plugin-react-router';

var browserPlugin = null;

const HttpRouterToken = createToken('HttpRouter');
const HttpHandlersToken = createToken('HttpHandlersToken');
const BodyParserOptionsToken = createToken('BodyParserOptionsToken');

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-env node */
function getHttpHandler(currentPath, method, paths, handlers) {
  const {
    path,
    params = {},
    isExact
  } = matchPath(currentPath, {
    path: paths
  }) || {};
  const handler = handlers[path] && handlers[path][method];

  if (path && handler && isExact) {
    if (typeof handler !== 'function') {
      throw new Error(`Missing/incorrect handler registered to ${method} of ${path}.`);
    }

    return [handler, params];
  }

  return [];
}
const plugin = createPlugin({
  deps: {
    bodyParserOptions: BodyParserOptionsToken.optional,
    handlers: HttpHandlersToken
  },
  provides: ({
    handlers,
    bodyParserOptions
  }) => {
    if (!handlers || typeof handlers !== 'object') {
      throw new Error('Missing/incorrect handlers registered to HttpHandlersToken');
    }

    const parseBody = bodyParser(bodyParserOptions);
    const paths = Object.keys(handlers);
    const from = memoize(ctx => new Promise(function ($return, $error) {
      let path, method, query, handler, params;
      ({
        path,
        method,
        query
      } = ctx);
      [handler, params] = getHttpHandler(path, method, paths, handlers);

      if (typeof handler !== 'function') {
        return $return(null);
      }

      return Promise.resolve(parseBody(ctx, () => Promise.resolve())).then(function ($await_3) {
        try {
          return $return(() => handler(_objectSpread({}, params, query, ctx.request.body), ctx));
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }, $error);
    }));
    return {
      from
    };
  },
  middleware: (deps, service) => (ctx, next) => new Promise(function ($return, $error) {
    var $Try_1_Post = function () {
      try {
        return $return(next());
      } catch ($boundEx) {
        return $error($boundEx);
      }
    };

    var $Try_1_Catch = function (error) {
      try {
        ctx.body = {
          status: 'failure',
          data: {
            code: error.code,
            message: error.message,
            stack: error.stack,
            meta: error.meta
          }
        };
        return $Try_1_Post();
      } catch ($boundEx) {
        return $error($boundEx);
      }
    };

    try {
      let handler;
      return Promise.resolve(service.from(ctx)).then(function ($await_4) {
        try {
          handler = $await_4;

          if (typeof handler === 'function') {
            return Promise.resolve(handler()).then(function ($await_5) {
              try {
                ctx.body = $await_5;
                return $If_2.call(this);
              } catch ($boundEx) {
                return $Try_1_Catch($boundEx);
              }
            }.bind(this), $Try_1_Catch);
          }

          function $If_2() {
            return $Try_1_Post();
          }

          return $If_2.call(this);
        } catch ($boundEx) {
          return $Try_1_Catch($boundEx);
        }
      }.bind(this), $Try_1_Catch);
    } catch (error) {
      $Try_1_Catch(error);
    }
  })
});

var index = browserPlugin;

export default index;
export { BodyParserOptionsToken, HttpHandlersToken, HttpRouterToken };
//# sourceMappingURL=browser.es2015.es.js.map
